@@ -23,86 +23,102 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 #include <string.h>
 #include <ctype.h>
 #include <unistd.h>
+
 #include "tty.h"
 #include "config.h"
+#include "misc.h"
 
 
+static FILE *fileptr;
+static int   sectionptr;
+static char  line[MAXLINE];
 
-configuration *configuration_init(char *filename)
+
+int configuration_init(char *file_name)
 {
-    configuration *this = (configuration *)malloc(sizeof(configuration)); 
-    this->file = fopen(filename, "r");
-    this->status = this->file != NULL;
-    return this;
-}
+    if (file_name == NULL) return 0;
 
+    fileptr = fopen(file_name, "r");
 
-void configuration_end(configuration *this)
-{
-    if (this->file) fclose(this->file);
-}
+    if (fileptr == NULL) return 0;
 
-
-int configuration_getline(configuration *this)
-{
-    int len;
-    char *comment;
-
-    if (fgets(this->line, MAXLINE - 1, this->file) == NULL)
-	return 0;
-    if ((len = strlen(this->line)) == MAXLINE - 1)
-	fatal("line too long");
-    if (comment = strchr(this->line, ICS)) *comment = 0;
-    else this->line[len - 1] = 0;
+    sectionptr = -1;
     return 1;
 }
 
 
-int configuration_getsectionptr(configuration *this, char *section_name)
+void configuration_end(void)
 {
-    char buf[MAXLINE];
+    if (fileptr) fclose(fileptr);
+}
 
-    fseek(this->file, 0, SEEK_SET);
-    while (configuration_getline(this))
+
+static int configuration_getline(void)
+{
+    int len, c;
+    char *comment;
+
+    if (fgets(line, MAXLINE, fileptr) == NULL)
+	return 0;
+
+    if ((len = strlen(line)) == MAXLINE - 1)
     {
-	sscanf(this->line, "%s", buf);
-	if (strcmp(section_name, buf) == 0)
-	    return ftell(this->file);
+	fprintf(stderr, "%s: configuration: line too long. Truncated.\n",
+		program);
+	/* search the end of this big line */
+	for (;;)
+	{
+	    c = fgetc(fileptr);
+	    if (c == '\n' || c == EOF) break;
+	}
     }
-    return -1;
+
+    if (comment = strchr(line, ICS))
+	*comment = 0;
+    else
+	if (line[len - 1] == '\n')
+	    line[len - 1] = 0;
+
+    return 1;
 }
 
 
-int  configuration_getstatus(configuration *this)
+int configuration_section(char *section_name)
 {
-    return this->status; 
+    fseek(fileptr, 0, SEEK_SET);
+
+    while (configuration_getline())
+	if (strcmp(section_name, line) == 0)
+	    return sectionptr = ftell(fileptr);
+
+    return sectionptr = -1;
 }
 
 
-void configuration_getfielddata(configuration *this, int sectionptr,
-				char *field_name, char **dest,
-				int fields, int seek)
+void configuration_getvarinfo(char *var_name, char **dest,
+			      int fields, int seek)
 {
     int fld;
     char buf[MAXLINE], *ptr, *tmp;
 
-    if (seek) fseek(this->file, sectionptr, SEEK_SET);
+    if (seek) fseek(fileptr, sectionptr, SEEK_SET);
 
     if (fields == 1)
 	*dest = 0;
     else
 	memset(dest, 0, fields * sizeof(char *));
 
-    while (configuration_getline(this) && *this->line)
+    while (configuration_getline() && *line)
     {
 	*buf = 0;
-	sscanf(this->line, "%s", buf);
+	sscanf(line, "%s", buf);
 	if (seek == NO_SEEK) buf[15] = 0;	/* just in case */
 
 	if (!is_print(*buf)) return;
-	if (seek == NO_SEEK || strcmp(field_name, buf) == 0)
+
+	if (seek == NO_SEEK || strcmp(var_name, buf) == 0)
 	{
-	    if ((ptr = strchr(this->line, IAS)) && *++ptr)
+	    if ((ptr = strchr(line, IAS)) && *++ptr)
 	    {
 		for (dest[0] = ptr, fld = 1; *ptr && fld < fields; ptr++)
 		    if (*ptr == IFS)
@@ -114,6 +130,7 @@ void configuration_getfielddata(configuration *this, int sectionptr,
 		    }
 		if (ptr = strchr(ptr, IFS)) *ptr = 0;
 	    }
+
 	    for (fld = 0; fld < fields; fld++)
 		if (dest[fld])
 		{
@@ -123,10 +140,11 @@ void configuration_getfielddata(configuration *this, int sectionptr,
 		    *(tmp + 1) = 0;
 		    if (dest[fld][0] == 0) dest[fld] = NULL;
 		}
-	    if (seek == NO_SEEK) strcpy(field_name, buf);
+
+	    if (seek == NO_SEEK) strcpy(var_name, buf);
 	    return;
 	}
     }
 
-    if (seek == NO_SEEK) *field_name = 0;
+    if (seek == NO_SEEK) *var_name = 0;
 }
