@@ -22,37 +22,23 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 #define _CONFIG_H
 
 
-#include <stdio.h>
+#define MAXLINE		1024
 
 
-#define MAXLINE		1024	/* change this to get more directory entries */
 #define NO_SEEK		0
 #define DO_SEEK		1
 
-#define STATUS_BAD	0
-#define STATUS_OK	1
 
 #define IFS  		';'	/* internal field separator */
 #define ICS		'#'	/* internal comment separator */
-#define IAS		'='	/* internal assignment separator */
+#define IAS		'='	/* internal assignment operator */
 
 
-typedef struct
-{
-    FILE *file;
-    int status;
-    char line[MAXLINE];
-} configuration;
-
-
-int  configuration_getline(configuration *this);
-configuration *configuration_init(char *filename);
-void configuration_end(configuration *this);
-int  configuration_getsectionptr(configuration *this, char *section_name);
-int  configuration_getstatus(configuration *this);
-void configuration_getfielddata(configuration *this, int sectionptr,
-				char *field_name, char **dest,
-				int fields, int seek);
+int  configuration_init(char *file_name);
+void configuration_end(void);
+int  configuration_section(char *section_name);
+void configuration_getvarinfo(char *var_name, char **dest,
+			      int fields, int seek);
 
 
 #endif	/* _CONFIG_H */
