@@ -20,22 +20,26 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 
 #include <stdio.h>
 #include <string.h>
-#include <malloc.h>
+
+#include "xmalloc.h"
+#include "config.h"
+#include "history.h"
 #include "tty.h"
 #include "window.h"
 #include "edit.h"
+#include "tilde.h"
+#include "misc.h"
 
 
-extern int  LinuxConsole;
-extern int  ColorMonitor;
-extern char cSection[];
-extern char bwSection[];
+extern int  AnsiColorSequences;
+
+
+#define MAX_INPUT_HISTORY	512	/* big enough I suppose ... */
 
-static char etemp[256];
 
 #define EDIT_FIELDS	3
 
-static char EditFields[EDIT_FIELDS][24] =
+static char *EditFields[EDIT_FIELDS] =
 {
     "CommandLineForeground",
     "CommandLineBackground",
@@ -47,65 +51,64 @@ static int EditColors[EDIT_FIELDS] =
 {
     WHITE, BLACK, ON
 };
-#else
+#else	/* not HAVE_LINUX */
 static int EditColors[EDIT_FIELDS] =
 {
     WHITE, BLACK, ON
 };
-#endif
+#endif	/* not HAVE_LINUX */
 
 #define CommandLineForeground	EditColors[0]
 #define CommandLineBackground	EditColors[1]
 #define CommandLineBrightness	EditColors[2]
 
 
-edit *edit_init(int _columns, int _begin_y, configuration *config)
+edit *edit_init(int _columns, int _begin_y)
 {
-    char *data, *section;
-    int sectionptr, index, i;
+    char *data;
+    int index, i;
 
-    edit *this        = (edit *)malloc(sizeof(edit));
+    edit *this        = (edit *)xmalloc(sizeof(edit));
     this->columns     = _columns;
-    this->records     = this->current_record = 0;
-    this->buf         = (char *)malloc(EDIT_BUFLEN);
-    this->history_buf = (char *)malloc(EDIT_HISTBUFLEN);
+    this->buf         = xmalloc(EDIT_BUFLEN + 1);
 
-    memset(this->history_buf, 0, EDIT_HISTBUFLEN);
     edit_reset(this);
     this->win = window_init(0, _begin_y, 1, _columns);
 
-    if (configuration_getstatus(config) == STATUS_OK)
-    {
-	section = (LinuxConsole && ColorMonitor) ? cSection : bwSection;
-	sectionptr = configuration_getsectionptr(config, section);
+    use_section("[UIT-Setup]");
 
-        if (sectionptr != -1)
-            for (i = 0; i < EDIT_FIELDS; i++)
-            {
-                configuration_getfielddata(config, sectionptr, EditFields[i],
-					   &data, 1, DO_SEEK);
-                if (!data || (index = tty_getcolorindex(data)) == -1)
-                    fprintf(stderr, "Invalid %s (%s).\n", EditFields[i], data);
-	        else
-                    EditColors[i] = index;
-            }
-    }
+    configuration_getvarinfo("HistoryFile", &data, 1, DO_SEEK);
+    this->history_file = tilde_expand(data ? data : "~/.uithistory");
 
+    use_section(AnsiColorSequences ? cSection : bwSection);
+
+    get_colorset_var(EditColors, EditFields, EDIT_FIELDS);
+
+    using_history();
+    read_history(this->history_file);
+    while (next_history());
+    stifle_history(MAX_INPUT_HISTORY);
     return this;
 } 
 
 
 void edit_end(edit *this)
 {
-    free(this->buf); 
-    free(this->history_buf);
+    write_history(this->history_file);
+    free(this->buf);
     free(this->win);
 }
 
 
-char *edit_gettext(edit *this, char *dest)
+/*
+ * dest is a pointer to a NULL pointer or a pointer to a pointer allocated
+ * with xmalloc(). See also the edit_gets() comment. 
+ */
+
+char *edit_gettext(edit *this, char **dest)
 {
-    return strcpy(dest, this->buf + this->static_size);
+    *dest = xrealloc(*dest, strlen(this->buf + this->static_size) + 1);
+    return strcpy(*dest, this->buf + this->static_size);
 }
 
 
@@ -115,23 +118,23 @@ void edit_reset(edit *this)
 }
 
 
-void edit_putch(edit *this, char key)
+int edit_putch(edit *this, char key)
 {
     tty_status status;
 
-    if (key == KEY_BACKSPACE && this->index == this->static_size) return;
+    if (key == key_BACKSPACE && this->index == this->static_size) return;
 
-    if (key != KEY_BACKSPACE && this->index == EDIT_BUFLEN - 1)
+    if (key != key_BACKSPACE && this->index == EDIT_BUFLEN)
     {
 	tty_beep();
-	return;
+	return 0;
     }
-         
-    if (key == KEY_BACKSPACE)
+
+    if (key == key_BACKSPACE)
         this->index--;
     else
     {
-        if (!is_print(key)) return;
+        if (!is_print(key)) return 0;
         this->buf[this->index++] = key;
     }
 
@@ -145,7 +148,7 @@ void edit_putch(edit *this, char key)
 	tty_foreground(CommandLineForeground);
 	tty_background(CommandLineBackground);
 
-	if (key == KEY_BACKSPACE)
+	if (key == key_BACKSPACE)
 	{
 	    window_cursormove_notify(this->win, 0, this->index);
 	    window_putch(' ');
@@ -162,6 +165,8 @@ void edit_putch(edit *this, char key)
     }
     else
         edit_update(this);
+
+    return 1;
 }
 
 
@@ -205,6 +210,7 @@ void edit_delete_last_word(edit *this)
 
 void edit_update(edit *this)
 {
+    char *temp;
     unsigned len;
     tty_status status;
 
@@ -213,14 +219,18 @@ void edit_update(edit *this)
     tty_bright(CommandLineBrightness);
     tty_foreground(CommandLineForeground);
     tty_background(CommandLineBackground);
-    
-    memset(etemp, ' ', this->columns);
+
+    temp = xmalloc(this->columns);
+
+    memset(temp, ' ', this->columns);
     len = (this->index >= this->columns) ? this->index - this->columns + 1 : 0;
-    memcpy(etemp, this->buf + this->static_size + len,
+    memcpy(temp, this->buf + this->static_size + len,
 	   this->index - this->static_size - len);
     window_cursormove_notify(this->win, 0, 0);
     window_write(this->buf, this->static_size);
-    window_write(etemp, this->columns - this->static_size);
+    window_write(temp, this->columns - this->static_size);
+
+    free(temp);
 
     tty_restore(&status);
 }
@@ -248,23 +258,41 @@ void edit_del(edit *this)
 }
 
 
-void edit_puts(edit *this, char *str)
+int edit_puts(edit *this, char *str)
 {
-    strncpy(this->buf + this->index, str, EDIT_BUFLEN - this->static_size - 1);
-    this->index = strlen(this->buf);
+    if (this->index + strlen(str) <= EDIT_BUFLEN)
+    {
+	strcpy(this->buf + this->index, str);
+	this->index = strlen(this->buf);
+	return 1;
+    }
+    else
+    {
+	tty_beep();
+	return 0;
+    }
 }
 
 
-char *edit_gets(edit *this, char *static_text, char *dest, char *default_str)
+/*
+ * WARNING: dest *must* be a pointer to a NULL pointer or a pointer to a
+ * pointer allocated with xmalloc. In the first case, edit_gets() will
+ * return a string allocated with xmalloc(). In the second case, edit_gets()
+ * will reallocate the pointer as needed using xrealloc. You should free
+ * this pointer yourself.
+ */
+
+char *edit_gets(edit *this, char *static_text, char **dest, char *default_str)
 {
     char key;
     struct key_struct *ks;
     int oldindex = this->index, oldstatic_size = this->static_size;
-    char oldbuf[EDIT_BUFLEN];
+    char *oldbuf;
 
+    oldbuf = xmalloc(strlen(this->buf) + 1);
     strcpy(oldbuf, this->buf);
     edit_reset(this);
-    edit_puts(this, static_text);
+    if (static_text) edit_puts(this, static_text);
     edit_eos(this);
     if (default_str) edit_puts(this, default_str);
     edit_update(this);
@@ -274,74 +302,105 @@ char *edit_gets(edit *this, char *static_text, char *dest, char *default_str)
     {
 	ks  = tty_getkey(NULL);
 	key = ks->key_seq[0];
-	if (ks->aux_data == NULL && (is_print(key) || key == KEY_BACKSPACE))
+	if (ks->aux_data == NULL && (is_print(key) || key == key_BACKSPACE))
 	    edit_putch(this, key);
 	else
 	    break;
     }
-    
-    edit_gettext(this, dest);
+
+    if (key == key_ENTER) edit_gettext(this, dest);
 
     strcpy(this->buf, oldbuf);
+    free(oldbuf);
     this->index = oldindex;
     this->static_size = oldstatic_size;
     edit_update(this);
     edit_setcursor(this);
-    return (key == KEY_ENTER) ? dest : NULL;
+    return (key == key_ENTER) ? *dest : NULL;
 }
 
 
 void edit_history(edit *this, int dir)
 {
+    static int browsing = 0;
+    static int last_history_pos;
+    HIST_ENTRY *hist;
+
     switch (dir)
     {
         case EDIT_PREVIOUS:
 
-	    if (this->current_record) this->current_record--;
-	    strncpy(this->buf + this->static_size,
-		    this->history_buf + this->current_record * EDIT_RECLEN,
-		    EDIT_BUFLEN - this->static_size - 1);
-	    this->index = strlen(this->buf);
-	    edit_update(this);
-            break;
-             
+	    if (!browsing)
+	    {
+		browsing = 1;
+		last_history_pos = where_history();
+	    }
+
+	    if ((hist = previous_history()))
+	    {
+		if (this->static_size + strlen(hist->line) <= EDIT_BUFLEN)
+		    strcpy(this->buf + this->static_size, hist->line);
+		else
+		{
+		    this->buf[this->static_size] = 0;
+		    tty_beep();
+		}
+
+		this->index = strlen(this->buf);
+		edit_update(this);
+	    }
+
+	    break;
+
         case EDIT_NEXT:
 
-            if (this->current_record < this->records)
-            {
-                strncpy(this->buf + this->static_size,
-			this->history_buf +
-			++this->current_record * EDIT_RECLEN,
-			EDIT_BUFLEN - this->static_size - 1);
+	    if (!browsing)
+	    {
+		browsing = 1;
+		last_history_pos = where_history();
+	    }
+
+	    if ((hist = next_history()))
+	    {
+		if (this->static_size + strlen(hist->line) <= EDIT_BUFLEN)
+		    strcpy(this->buf + this->static_size, hist->line);
+		else
+		{
+		    this->buf[this->static_size] = 0;
+		    tty_beep();
+		}
+
 		this->index = strlen(this->buf);
 		edit_update(this);
 	    }
 	    else
-	        edit_del(this);
+		edit_del(this);
+
 	    break;
-             
+
         case EDIT_RECORD:
 
-            if ((this->records &&
-		 strcmp(&this->history_buf[(this->records - 1) * EDIT_RECLEN],
-			this->buf + this->static_size) == 0) ||
-		this->buf[this->static_size] == 0)
+	    if (browsing)
 	    {
-		this->current_record = this->records;
-		break;
+		history_set_pos(last_history_pos);
+		browsing = 0;
 	    }
 
-	    if (this->records == EDIT_HISTRECS)
+	    if ((hist = previous_history()))
 	    {
-		memcpy(this->history_buf, this->history_buf + EDIT_RECLEN,
-		       EDIT_HISTBUFLEN - EDIT_RECLEN);
-		this->records--;
+		if (strcmp(this->buf + this->static_size, hist->line) != 0)
+		{
+		    add_history(this->buf + this->static_size);
+		    next_history();
+		}
 	    }
-	    strcpy(this->history_buf + this->records * EDIT_RECLEN,
-		   this->buf + this->static_size);
-	    this->current_record = ++this->records;
+	    else
+		add_history(this->buf + this->static_size);
+
+	    next_history();
+
 	    break;
-             
+
         default:
 
             break;
