@@ -22,13 +22,10 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 #define _EDIT_H
 
 
-#include "config.h"
+#include "window.h"
 
 
-#define EDIT_BUFLEN	0xFF
-#define EDIT_HISTBUFLEN	0x2000
-#define EDIT_RECLEN	0x100
-#define EDIT_HISTRECS   (EDIT_HISTBUFLEN / EDIT_RECLEN)
+#define EDIT_BUFLEN	0x800
 #define EDIT_RECORD	0
 #define EDIT_PREVIOUS	1
 #define EDIT_NEXT	2
@@ -38,17 +35,18 @@ typedef struct
 {
     window *win;
     unsigned index, columns;
-    unsigned static_size, records, current_record;
-    char *buf, *history_buf;
+    unsigned static_size;
+    char *buf;
+    char *history_file;
 } edit;
 
 
-edit *edit_init(int columns, int begin_y, configuration *config);
+edit *edit_init(int columns, int begin_y);
 void  edit_end(edit *this);
-void  edit_putch(edit *this, char key);
-void  edit_puts(edit *this, char *str);
-char *edit_gets(edit *this, char *static_text, char *dest, char *default_str);
-char *edit_gettext(edit *this, char *dest);
+int   edit_putch(edit *this, char key);
+int   edit_puts(edit *this, char *str);
+char *edit_gets(edit *this, char *static_text, char **dest, char *default_str);
+char *edit_gettext(edit *this, char **dest);
 void  edit_reset(edit *this);
 void  edit_del(edit *this);
 void  edit_setcursor(edit *this);
