@@ -24,11 +24,10 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 
 #include <dirent.h>
 #include <limits.h>
+#include <stddef.h>
+#include <unistd.h>
+#include <sys/types.h>
 #include "edit.h"
-#include "config.h"
-
-
-#define MAX_ENTRIES	0x400
 
 
 /* Panel actions */
@@ -104,37 +103,48 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 #define SOCKET_ENTRY		 4
 
 
+#define ACTIVE_PANEL_MARKER	'>'
+#define INACTIVE_PANEL_MARKER	'*'
+
+
 typedef struct
 {
-    char *name;
-    int  size, mode, uid, gid;
-    char owner[8];
-    char group[8];
-    char date[15];
-    time_t mtime;
-    unsigned char selected   : 1,
+    char	 *name;
+    size_t	  size;
+    mode_t	  mode;
+    uid_t	  uid;
+    gid_t	  gid;
+    time_t	  mtime;
+    char	  owner[8];
+    char	  group[8];
+    char	  date[16];
+    unsigned      selected   : 1,
     		  type       : 4,
     		  executable : 1;
-} _dir_entry;
+} dir_entry_struct;
 
 
 typedef struct
 {
+    DIR *dir;
+    edit *edt;
+    window *win;
+#ifdef HAVE_LINUX
+    int msdosfs;
+#endif	/* HAVE_LINUX */
     int on_screen;
     int current_entry;
+    char *path, *temp;
     int first_on_screen;
-    int last_index, found_selection, end;
-    int lines, columns, begin_x, begin_y;
-    int display_mode, sort_method, chkdest;
-    int entries, focus, selected_files;
     unsigned int maxname;
-    edit *edt;
-    DIR *dir;
-    window *win;
-    char path[PATH_MAX];
-    _dir_entry dir_entry[MAX_ENTRIES];
+    dir_entry_struct *dir_entry;
+    int lines, columns, begin_x, begin_y;
+    int last_index, found_selection, end;
+    int display_mode, sort_method, chkdest;
+    int entries, focus, selected_files, pathlen;
 } panel;
 
+
 void panel_recover(panel *this);
 void panel_update(panel *this);  
 void panel_update_frame(panel *this);
@@ -145,13 +155,12 @@ void panel_update_entry(panel *this, int entry);
 void panel_trunc_fname(char *fname, char *dest, int len);
 int  panel_move(panel *this, char *from, char *to);
 int  panel_copy(panel *this, char *src, char *dest, int mode);
-int  panel_verify_name(char *file_name);
 int  panel_getnext(panel *this);
 int  panel_getindex(panel *this, char *str);
 int  panel_getdirinfo(panel *this, char *directory, int verify);
 
 panel *panel_init(int lines, int columns, int begin_x, int begin_y, char *path,
-		  edit *edt, int *UserHeartAttack, configuration *config);
+		  edit *edt, int *UserHeartAttack);
 void  panel_end(panel *this);
 void  panel_setfocus(panel *this, int status, panel *link);
 void  panel_nooptimizations(panel *this);
