@@ -22,16 +22,12 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 #define _STATUS_H
 
 
-#include "config.h"
-
-
 #define MSG_OK		0
 #define MSG_WARNING	1
 #define MSG_ERROR	2
 
 
-void status_init(int columns, int begin_y,
-		 char *def_msg, configuration *config);
+void status_init(int columns, int begin_y, char *def_msg);
 void status_end(void);
 int  status(char *msg_name, int wait, int sound, int restore, int msg_type);
 
