@@ -21,53 +21,166 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <ctype.h>
 #include <fcntl.h>
 #include <string.h>
 #include <termios.h>
 #include <sys/ioctl.h>
+
+#ifdef HAVE_LINUX
+#include <sys/stat.h>
+#endif /* HAVE_LINUX */
+
+#include "xmalloc.h"
 #include "tty.h"
-#include "termcap.h"
+#include "misc.h"
+
+
+#ifndef HAVE_CONST
+#define const
+#endif	/* HAVE_CONST */
+
+
+int tgetent(void *__buffer, const char *__termtype);
+int tputs(const char *__string, int __nlines, int (*outfun)());
+char *tgetstr(const char *__name, char **__area);
+char *tgoto(const char *__cstring, int __hpos, int __vpos);
 
 
 #define TTY_INPUT	0
 #define TTY_OUTPUT	1
 
 
-int    tty_kbdmode = 0;	/* If tty_mode == 1 single characters are inserted
-			   in the linked list. This feature is used by uitps
-			   (no command line ... ) */
+/*
+ * If tty_kbdmode == 1 single characters are inserted in the linked list.
+ * This feature is used by uitps (no command line ... )
+ */
+static int tty_kbdmode = 0; 
 
-static struct termios oldterm;
-static struct termios newterm;
+static struct termios old_term;
+static struct termios new_term;
 
 static int tty_perm;
 static int tty_cursor_x;
 static int tty_cursor_y;
 static int rows, columns;
 static int tty_cursormove_notified;
-static int fg_color, bg_color, br_status, rv_status;
+static int fg_color, bg_color, br_status, rv_status, cursor_status;
 
 static unsigned char tty_current_attribute;
 
-static unsigned char *tty_screen;
-static unsigned char *tty_attributes;
+static unsigned char *tty_screen = NULL;
+static unsigned char *tty_attributes = NULL;
 
 
-extern int LinuxConsole;
+static unsigned char ansi_foreground[] = { 0x1b, '[', '3', '0', 'm' };
+static unsigned char ansi_background[] = { 0x1b, '[', '4', '0', 'm' };
+
+#ifdef HAVE_LINUX
+static unsigned char lc_attr_off[] = { 0x1b, '[', 'm' };
+static unsigned char lc_defaults[] = { 0x1b, '[', '0', 'm' };
+#endif /* HAVE_LINUX */
+
+
+extern int AnsiColorSequences;
+
+
+#ifdef HAVE_LINUX
+int LinuxConsole;
+#endif /* HAVE_LINUX */
 
 
 struct key_struct *key_list_head = NULL;
 struct key_struct *current_key;
 struct key_struct default_key;
 
-static char tty_name[64] = "";
-static int  tty_name_len;
+static char tty_buf[MAX_CAPABILITY_LEN];
+static int  tty_index = 0;
+
+static char term_buf[2048];
+static char vt100[] = "vt100";
+
+int tty_mode = TTY_CANONIC;
+
+char *tty_type;
+
+char PC;	/* for tputs */
+char *BC;	/* for tgoto */
+char *UP;
+
+#ifdef HAVE_LINUX
+speed_t ospeed;
+#else	/* not HAVE_LINUX */
+short ospeed;
+#endif	/*not HAVE_LINUX */
+
+
+typedef struct
+{
+    char *capability;
+    char *name;
+    int  indispensable;
+} TTY_CAPABILITY;
+
+
+#define TTY_USED_CAPABILITIES	10
+
+static TTY_CAPABILITY tty_capability[TTY_USED_CAPABILITIES] =
+{
+    { NULL, "me", 0 },		/* turn off all attributes */
+    { NULL, "mr", 0 },		/* turn on reverse video mode */
+    { NULL, "md", 0 },		/* turn on bold	*/
+    { NULL, "vi", 0 },		/* make the cursor invisible */
+    { NULL, "ve", 0 },		/* make the cursor appear normal */
+    { NULL, "cl", 1 },		/* clear screen and home the cursor */
+    { NULL, "cm", 1 },		/* move the cursor */
+    { NULL, "pc", 0 },		/* padding character */
+    { NULL, "up", 0 },		/* up one line */
+    { NULL, "le", 0 },		/* move left one space */
+};
+
+
+#define TTY_ATTRIBUTES_OFF	(tty_capability[0].capability)
+#define TTY_REVERSE_ON		(tty_capability[1].capability)
+#define TTY_BRIGHT_ON		(tty_capability[2].capability)
+#define TTY_CURSOR_OFF		(tty_capability[3].capability)
+#define TTY_CURSOR_ON		(tty_capability[4].capability)
+#define TTY_CLEAR_SCREEN	(tty_capability[5].capability)
+#define TTY_CURSOR_MOVE		(tty_capability[6].capability)
+#define TTY_PAD_CHAR		(tty_capability[7].capability)
+#define TTY_UP_ONE_LINE		(tty_capability[8].capability)
+#define TTY_LEFT_ONE_SPACE	(tty_capability[9].capability)
+
+
+#define TTY_ATTRIBUTES_OFF_NAME	(tty_capability[0].name)
+#define TTY_REVERSE_ON_NAME	(tty_capability[1].name)
+#define TTY_BRIGHT_ON_NAME	(tty_capability[2].name)
+#define TTY_CURSOR_OFF_NAME	(tty_capability[3].name)
+#define TTY_CURSOR_ON_NAME	(tty_capability[4].name)
+#define TTY_CLEAR_SCREEN_NAME	(tty_capability[5].name)
+#define TTY_CURSOR_MOVE_NAME	(tty_capability[6].name)
+#define TTY_PAD_CHAR_NAME	(tty_capability[7].name)
+#define TTY_UP_ONE_LINE_NAME	(tty_capability[8].name)
+#define TTY_LEFT_ONE_SPACE_NAME	(tty_capability[9].name)
+
+
+#ifdef USE_TERMCAP
+
+static char term_database[] = "termcap";
+static char term_env[]	    = "TERMCAP";
+
+#else	/* not USE_TERMCAP (USE_TERMINFO) */
+
+static char term_database[] = "terminfo";
+static char term_env[]	    = "TERMINFO";
+
+#endif	/* not USE_TERMCAP (USE_TERMINFO) */
 
 
 void fatal(char *);
 
 
-static char colors[10][10] = 
+static char *colors[10] = 
 {
     "BLACK",
     "RED",
@@ -182,14 +295,12 @@ char key_ctrl_tbl[0x5f] =
 };
 
 
-extern char termcap[TTY_MAXCAP][16];
-extern char termcap_len[TTY_MAXCAP];
-
-
 #define NO	0
 #define YES	1
 
 
+#define LINUX_VC_MAJOR	4
+
 static int  keyno    = 0;
 static int  keyindex = 0;
 static char keybuf[1024];
@@ -197,54 +308,59 @@ static char keybuf[1024];
 
 void tty_startup(void)
 {
-    strncpy(tty_name, ttyname(1), sizeof(tty_name) - 1);
-    tty_name[sizeof(tty_name) - 1] = 0;
-    tty_name_len = strlen(tty_name);
+    if (!isatty(TTY_INPUT) || !isatty(TTY_OUTPUT))
+	fatal("only standard error can be redirected");
 
-    tty_screen     = (unsigned char *)calloc(columns * rows,
-    					     sizeof(unsigned char));
-    tty_attributes = (unsigned char *)calloc(columns * rows,
-    					     sizeof(unsigned char));
+    tty_screen     = (unsigned char *)xcalloc(columns * rows,
+    					      sizeof(unsigned char));
+    tty_attributes = (unsigned char *)xcalloc(columns * rows,
+    					      sizeof(unsigned char));
     tty_current_attribute = 0;
+
+    tcgetattr(TTY_OUTPUT, &old_term);
+}
+
+
+void tty_kbdinit(int kbd_mode)
+{
+    default_key.key_seq  = xmalloc(16);
+    default_key.aux_data = NULL;
+    default_key.next     = NULL;
+    tty_kbdmode 	 = kbd_mode;
+}
+
+
+void tty_set_mode(int mode)
+{
+    if (mode == TTY_NONCANONIC)
+    {
+	new_term = old_term;
+	new_term.c_iflag &= ~(ICRNL | IGNCR | INLCR | IGNBRK | BRKINT);
+	new_term.c_oflag &= ~OPOST;
+	new_term.c_lflag |= ISIG | NOFLSH;
+	new_term.c_lflag &= ~(ICANON | ECHO);
+	new_term.c_cc[VMIN]  = 1;
+	new_term.c_cc[VTIME] = 0;
+	tcsetattr(TTY_OUTPUT, TCSADRAIN, &new_term);
+	ospeed = cfgetospeed(&new_term);
+    }
+    else
+	tcsetattr(TTY_OUTPUT, TCSADRAIN, &old_term);
+
+    tty_mode = mode;
+    tty_defaults();
 }
 
 
 void tty_exit(void)
 {
-    free(tty_screen);
-    free(tty_attributes);
-}
-
-
-void tty_init(void)
-{
-    tty_defaults();
-
-    tcgetattr(TTY_OUTPUT, &oldterm);
-    newterm = oldterm;
-    newterm.c_iflag &= ~(ICRNL | IGNCR | INLCR | IGNBRK | BRKINT);
-    newterm.c_oflag &= ~OPOST;
-    newterm.c_lflag |= ISIG | NOFLSH;
-    newterm.c_lflag &= ~(ICANON | ECHO);
-    newterm.c_cc[VMIN]  = 1;
-    newterm.c_cc[VTIME] = 0;
-    tcsetattr(TTY_OUTPUT, TCSADRAIN, &newterm);
-}
-
-
-void tty_end(void)
-{
-    tcsetattr(TTY_OUTPUT, TCSADRAIN, &oldterm);
-    tty_defaults();
-}
-
-
-void tty_kbdinit(int mode)
-{
-    default_key.key_seq  = (char *)malloc(16);
-    default_key.aux_data = NULL;
-    default_key.next     = NULL;
-    tty_kbdmode 	 = mode;
+    if (tty_mode == TTY_NONCANONIC)
+    {
+	tty_set_mode(TTY_CANONIC);
+	tty_clrscr();
+    }
+    else
+	tty_defaults();
 }
 
 
@@ -271,12 +387,26 @@ char *tty_key_convert(char *key_seq)
 }
 
 
+int tty_out_char(int c)
+{
+    return tty_buf[tty_index++] = (char)c;
+}
+
+
+void tty_flush(void)
+{
+    write(TTY_OUTPUT, tty_buf, tty_index);
+    tty_index = 0;
+}
+
+
 void tty_clrscr(void)
 {
     memset(tty_screen, ' ', rows * columns * sizeof(unsigned char));
     memset(tty_attributes, tty_current_attribute,
     	   rows * columns * sizeof(unsigned char));
-    write(TTY_OUTPUT, termcap[TTY_CLRSCR], termcap_len[TTY_CLRSCR]);
+    tputs(TTY_CLEAR_SCREEN, rows, tty_out_char);
+    tty_flush();
 }
 
 
@@ -288,10 +418,13 @@ void tty_touch(void)
 
 void tty_cursormove(int y, int x)
 {
-    char msg[10];
+    int old_rv_status;
+    char *str;
+
+    str = tgoto(TTY_CURSOR_MOVE, x, y);
+    tputs(str, 1, tty_out_char);
+    tty_flush();
 
-    sprintf(msg, "\33[%d;%dH", y + 1, x + 1);
-    write(TTY_OUTPUT, msg, strlen(msg));
     tty_cursor_x = x;
     tty_cursor_y = y;
 }
@@ -307,19 +440,17 @@ void tty_cursormove_notify(int y, int x)
 
 void tty_foreground(int color)
 {
-#ifdef HAVE_LINUX
-    char str[] = "\x1b\x5b\x33\x30\x6d";
+    char str[16];
 
-    if (LinuxConsole)
+    memcpy(str, ansi_foreground, sizeof(ansi_foreground));
+
+    if (AnsiColorSequences == ON)
     {
 	str[3] += (fg_color = color);
-	write(TTY_OUTPUT, str, 5);
+	write(TTY_OUTPUT, str, sizeof(ansi_foreground));
     }
     else
 	tty_reverse((fg_color = color) != WHITE);
-#else
-    tty_reverse((fg_color = color) != WHITE);
-#endif
 
     tty_current_attribute = (tty_current_attribute & 0xF8) | fg_color;
 }
@@ -327,19 +458,17 @@ void tty_foreground(int color)
 
 void tty_background(int color)
 {
-#ifdef HAVE_LINUX
-    char str[] = "\x1b\x5b\x34\x30\x6d";
+    char str[16];
 
-    if (LinuxConsole)
+    memcpy(str, ansi_background, sizeof(ansi_background));
+
+    if (AnsiColorSequences == ON)
     {
 	str[3] += (bg_color = color);
-	write(TTY_OUTPUT, str, 5);
+	write(TTY_OUTPUT, str, sizeof(ansi_background));
     }
     else
 	tty_reverse((bg_color = color) != BLACK);
-#else
-    tty_reverse((bg_color = color) != BLACK);
-#endif
 
     tty_current_attribute = (tty_current_attribute & 0xC7) | (bg_color << 3);
 }
@@ -348,22 +477,32 @@ void tty_background(int color)
 void tty_bright(int status)
 {
     if (status == ON)
-	write(TTY_OUTPUT, termcap[TTY_BRIGHT_ON], termcap_len[TTY_BRIGHT_ON]);
-    else
     {
-#ifdef HAVE_LINUX
-	if (LinuxConsole)
-	    write(TTY_OUTPUT, "\x1b\x5b\x32\x32\x6d", 5);
-	else
-#endif
+	if (TTY_BRIGHT_ON)
 	{
-	    write(TTY_OUTPUT,
-	    	  termcap[TTY_BRIGHT_OFF],
-	    	  termcap_len[TTY_BRIGHT_OFF]);
-	    br_status = OFF;
-	    if (rv_status) tty_reverse(rv_status);
+	    tputs(TTY_BRIGHT_ON, 1, tty_out_char);
+	    tty_flush();
 	}
     }
+    else
+    {
+	if (TTY_ATTRIBUTES_OFF)
+	{
+	    tputs(TTY_ATTRIBUTES_OFF, 1, tty_out_char);
+	    tty_flush();
+	}
+	br_status = OFF;
+	/*
+	 * There is no terminal capability for bright off  (or bold off).
+	 * I am using the 'me' capability (where available)  which  turns
+	 * off all attributes so we must restore the reverse status after
+	 * that. There is no need to restore the  foreground & background
+	 * colors because I've  always  put  tty_bright(status)  *BEFORE*
+	 * tty_foreground(color) or tty_background(color).
+	 */
+	if (rv_status == ON) tty_reverse(ON);
+    }
+
     br_status = status;
 
     tty_current_attribute = (tty_current_attribute & 0xBF) | (br_status << 6);
@@ -375,42 +514,75 @@ void tty_bright(int status)
 void tty_reverse(int status)
 {
     if (status == ON)
-	write(TTY_OUTPUT,termcap[TTY_REVERSE_ON],termcap_len[TTY_REVERSE_ON]);
+    {
+	if (TTY_REVERSE_ON)
+	{
+	    tputs(TTY_REVERSE_ON, 1, tty_out_char);
+	    tty_flush();
+	}
+    }
     else
     {
-	write(TTY_OUTPUT,
-	      termcap[TTY_REVERSE_OFF],
-	      termcap_len[TTY_REVERSE_OFF]);
+	if (TTY_ATTRIBUTES_OFF)
+	{
+	    tputs(TTY_ATTRIBUTES_OFF, 1, tty_out_char);
+	    tty_flush();
+	}
 	rv_status = OFF;
-	if (br_status) tty_bright(br_status);
+	/* same comment as in tty_bright() */
+	if (br_status == ON) tty_bright(ON);
     }
+
     rv_status = status;
 }
 
 
 void tty_beep(void)
 {
-    char c = 7;	
+    char c = 7;
     write(TTY_OUTPUT, &c, 1);
 }
 
 
+void tty_cursor(int status)
+{
+    if (cursor_status = status)
+    {
+	if (TTY_CURSOR_ON)
+	{
+	    tputs(TTY_CURSOR_ON, 1, tty_out_char);
+	    tty_flush();
+	}
+    }
+    else
+    {
+	if (TTY_CURSOR_OFF)
+	{
+	    tputs(TTY_CURSOR_OFF, 1, tty_out_char);
+	    tty_flush();
+	}
+    }
+}
+
+
 void tty_save(tty_status *status)
 {
-    status->fg_color  = fg_color;
-    status->bg_color  = bg_color;
-    status->br_status = br_status;
-    status->rv_status = rv_status;
-    status->attribute = tty_current_attribute;
+    status->fg_color	  = fg_color;
+    status->bg_color	  = bg_color;
+    status->br_status	  = br_status;
+    status->rv_status	  = rv_status;
+    status->cursor_status = cursor_status;
+    status->attribute	  = tty_current_attribute;
 }
 
 
 void tty_restore(tty_status *status)
 {
+    tty_bright(status->br_status);
     tty_foreground(status->fg_color);
     tty_background(status->bg_color);
-    tty_bright(status->br_status);
     tty_reverse(status->rv_status);
+    tty_cursor(status->cursor_status);
     tty_current_attribute = status->attribute;
 }
 
@@ -418,24 +590,22 @@ void tty_restore(tty_status *status)
 void tty_defaults(void)
 {
 #ifdef HAVE_LINUX
-    if (LinuxConsole)
-	write(TTY_OUTPUT, "\x1b\x5b\x30\x6d", 4);  /* Linux console defaults */
+    if (LinuxConsole == ON)
+	write(TTY_OUTPUT, lc_defaults, sizeof(lc_defaults));
     else
-    {
-	write(TTY_OUTPUT,
-	      termcap[TTY_BRIGHT_OFF],
-	      termcap_len[TTY_BRIGHT_OFF]);
-	write(TTY_OUTPUT,
-	      termcap[TTY_REVERSE_OFF],
-	      termcap_len[TTY_REVERSE_OFF]);
-    }
-#else
-    write(TTY_OUTPUT, termcap[TTY_BRIGHT_OFF], termcap_len[TTY_BRIGHT_OFF]);
-    write(TTY_OUTPUT, termcap[TTY_REVERSE_OFF], termcap_len[TTY_REVERSE_OFF]);
-#endif
-    fg_color = WHITE;
-    bg_color = BLACK;
+#endif	/* HAVE_LINUX */
+	if (TTY_ATTRIBUTES_OFF)
+	{
+	    tputs(TTY_ATTRIBUTES_OFF, 1, tty_out_char);
+	    tty_flush();
+	}
+
+    tty_foreground(WHITE);
+    tty_background(BLACK);
     br_status = rv_status = OFF;
+
+    tty_cursor(ON);
+    
     tty_current_attribute = (br_status << 6) | (bg_color << 3) | fg_color;
 }
 
@@ -492,21 +662,8 @@ void tty_key_list_insert_sequence(struct key_struct **key,
 {
     struct key_struct *new_key;
 
-    new_key = (struct key_struct *)malloc(sizeof(struct key_struct));
-    if (new_key == NULL)
-    {
-	fprintf(stderr, "not enough memory\n");
-	return;
-    }
-
-    new_key->key_seq = (char *)malloc(strlen(key_seq) + 1);
-    if (new_key->key_seq == NULL)
-    {
-	fprintf(stderr, "not enough memory\n");
-	free(new_key);
-	return;
-    }
-
+    new_key = (struct key_struct *)xmalloc(sizeof(struct key_struct));
+    new_key->key_seq = xmalloc(strlen(key_seq) + 1);
     strcpy(new_key->key_seq, key_seq);
     new_key->aux_data = aux_data;
     new_key->next = *key;
@@ -558,6 +715,7 @@ void tty_key_search_restart(void)
 }
 
 
+/*
 void tty_key_list_delete(void)
 {
     struct key_struct *key, *next_key;
@@ -569,6 +727,7 @@ void tty_key_list_delete(void)
 	free(key);
     }
 }
+*/
 
 
 struct key_struct *tty_getkey(int *repeat_count)
@@ -588,19 +747,19 @@ struct key_struct *tty_getkey(int *repeat_count)
     {
 	if (c == '\n' || c == '\r')
 	{
-	    default_key.key_seq[0] = KEY_ENTER;
+	    default_key.key_seq[0] = key_ENTER;
 	    default_key.key_seq[1] = 0;
 	    return &default_key;
 	}
 
-	if (c == KEY_CTRL_H)
+	if (c == key_CTRL_H)
 	{
-	    default_key.key_seq[0] = KEY_BACKSPACE;
+	    default_key.key_seq[0] = key_BACKSPACE;
 	    default_key.key_seq[1] = 0;
 	    return &default_key;
 	}
 
-	if (is_print(c) || c == KEY_TAB || c == KEY_BACKSPACE)
+	if (is_print(c) || c == key_TAB || c == key_BACKSPACE)
 	{
 	    default_key.key_seq[0] = c;
 	    default_key.key_seq[1] = 0;
@@ -635,24 +794,35 @@ struct key_struct *tty_getkey(int *repeat_count)
 void tty_getsize(int *_columns, int *_rows)
 {
     char *env;
+
+#ifndef HAVE_SYSTEMV
     struct winsize winsz;
+#endif /* HAVE_SYSTEMV */
+
     int temp_rows, temp_columns;
 
     columns = rows = temp_columns = temp_rows = 0;
 
+#ifndef HAVE_SYSTEMV
     ioctl(TTY_OUTPUT, TIOCGWINSZ, &winsz);
+
     if (winsz.ws_col && winsz.ws_row)
     {
 	temp_columns = winsz.ws_col;
 	temp_rows    = winsz.ws_row;
     }
+#endif /* HAVE_SYSTEMV */
 
     if (env = getenv("COLUMNS")) sscanf(env, "%d", &columns);
     if (env = getenv("LINES"))   sscanf(env, "%d", &rows);
+
+#ifndef HAVE_SYSTEMV
     if (columns < 80) columns = temp_columns;
     if (rows    < 10) rows    = temp_rows;
-    if (columns < 80 || columns > 200) columns = 80;
-    if (rows    < 10 || rows    > 128) rows    = 24;
+#endif /* HAVE_SYSTEMV */
+
+    if (columns < 80 || columns > MAX_TTY_COLUMNS) columns = 80;
+    if (rows    < 10 || rows    > MAX_TTY_ROWS)    rows    = 24;
 
     *_columns = columns;
     *_rows    = rows;
@@ -664,17 +834,8 @@ void tty_getscreen(char *buf)
 #ifdef HAVE_LINUX
     int x, y, index;
 
-    if (LinuxConsole)
+    if (LinuxConsole == ON)
     {
-	if (tty_name[tty_name_len - 4] != 't' ||
-	    tty_name[tty_name_len - 3] != 't' ||
-	    tty_name[tty_name_len - 2] != 'y' ||
-	    !isdigit(tty_name[tty_name_len - 1]))
-	{
-	    tty_perm = 0;
-	    return;
-	}
-
 	buf[0] = 0;
 	buf[1] = tty_name[tty_name_len - 1] - '0';
 
@@ -683,7 +844,7 @@ void tty_getscreen(char *buf)
 	    buf[0] = buf[1] = 0;			
 	    
 	    /*
-	     * Old Linux bug: bad ioctl on console 8. I think this was
+	     * Old Linux bug: bad ioctl on console 8. I think this has been
 	     * fixed in the current Linux kernel, but the test won't hurt.
 	     */
 	    
@@ -693,8 +854,10 @@ void tty_getscreen(char *buf)
 		return;
 	    }
 	}
+
 	tty_perm = 1;
 	index = 2 + rows * columns;
+
 	for (y = rows - 1; y >= 0; y--)
 	    for (x = columns - 1; x >= 0; x--)
 	        if (buf[--index] != ' ')
@@ -705,7 +868,7 @@ void tty_getscreen(char *buf)
 	buf[1] = 0; 
 	tty_cursormove(y + 1, 0);
     }
-#endif
+#endif	/* HAVE_LINUX */
 }
 
 
@@ -715,7 +878,7 @@ void tty_putscreen(char *buf)
     tty_cursormove(0, 0);
 
 #ifdef HAVE_LINUX
-    if (LinuxConsole)
+    if (LinuxConsole == ON)
     {
 	if (tty_perm)
 	{
@@ -734,10 +897,10 @@ void tty_putscreen(char *buf)
 	tty_defaults();
 	tty_clrscr();
     }
-#else
+#else	/* not HAVE_LINUX */
     tty_defaults();
     tty_clrscr();
-#endif
+#endif	/* not HAVE_LINUX */
 }
 
 
@@ -750,3 +913,118 @@ int tty_getcolorindex(char *colorname)
             return (i < 8) ? i : (i - 8);
     return -1;
 }
+
+
+
+void tty_get_capabilities(void)
+{
+#ifdef HAVE_LINUX
+    struct stat statbuf;
+#endif /* HAVE_LINUX */
+    char *capability_buf, *tmp;
+    int err, i, term_errors = 0;
+    char *termtype = getenv("TERM");
+
+#ifdef HAVE_LINUX
+    fstat(TTY_OUTPUT, &statbuf);
+    if ((statbuf.st_rdev >> 8) == LINUX_VC_MAJOR &&
+        ((unsigned)(statbuf.st_rdev & 0xFF)) <= 8)
+	LinuxConsole = ON;
+    else
+	LinuxConsole = OFF;
+#endif /* HAVE_LINUX */
+
+    if (termtype == NULL)
+    {
+	fprintf(stderr, "%s: can't find the TERM environment variable, ",
+		program);
+	goto switch_to_vt100;
+    }
+
+    if (strlen(termtype) > 63)
+    {
+	fprintf(stderr, "%s: the TERM environment variable is too long, ",
+		program);
+      switch_to_vt100:
+	fprintf(stderr, "trying vt100 ...\n");
+	termtype = vt100;
+    }
+
+    err = tgetent(term_buf, termtype);
+    
+    if (err == -1)
+    {
+	fprintf(stderr, "%s: can't find the %s database.\n",
+		program, term_database);
+	fprintf(stderr, "%s: check your %s environment variable ...\n",
+		program, term_env);
+	exit(1);
+    }
+
+    if (err == 0)
+    {
+	fprintf(stderr,
+		"%s: can't find the terminal type %s in the %s database.\n",
+		program, termtype, term_database);
+	exit(1);
+    }
+
+    tty_type = xmalloc(strlen(termtype) + 1);
+    strcpy(tty_type, termtype);
+
+#ifdef USE_TERMCAP
+    capability_buf = xmalloc(strlen(term_buf));
+#else	/* not USE_TERMCAP (USE_TERMINFO) */
+    /* we are using only 10 capabilities, so we don't need much space */
+    capability_buf = xmalloc(512);
+#endif	/* not USE_TERMCAP (USE_TERMINFO) */
+
+    tmp = tgetstr(TTY_PAD_CHAR_NAME, &capability_buf);
+    PC = tmp ? *tmp : 0;
+
+    BC = tgetstr(TTY_LEFT_ONE_SPACE_NAME, &capability_buf);
+    UP = tgetstr(TTY_UP_ONE_LINE_NAME,    &capability_buf);
+
+    if (BC == NULL || UP == NULL)
+	BC = UP = NULL;
+
+    TTY_ATTRIBUTES_OFF = tgetstr(TTY_ATTRIBUTES_OFF_NAME, &capability_buf);
+    TTY_REVERSE_ON     = tgetstr(TTY_REVERSE_ON_NAME,	  &capability_buf);
+    TTY_BRIGHT_ON      = tgetstr(TTY_BRIGHT_ON_NAME,	  &capability_buf);
+
+    if (TTY_ATTRIBUTES_OFF == NULL)
+    {
+#ifdef HAVE_LINUX
+	if (LinuxConsole)
+	    TTY_ATTRIBUTES_OFF = lc_attr_off;	/* I *really* hate this ! */
+	else
+#endif /* HAVE_LINUX */
+	    TTY_REVERSE_ON = TTY_BRIGHT_ON = NULL;
+    }
+
+    TTY_CURSOR_OFF = tgetstr(TTY_CURSOR_OFF_NAME, &capability_buf);
+    TTY_CURSOR_ON  = tgetstr(TTY_CURSOR_ON_NAME,  &capability_buf);
+
+    if (TTY_CURSOR_OFF == NULL || TTY_CURSOR_ON == NULL)
+	TTY_CURSOR_ON = TTY_CURSOR_OFF = NULL;
+
+    TTY_CLEAR_SCREEN = tgetstr(TTY_CLEAR_SCREEN_NAME, &capability_buf);
+    TTY_CURSOR_MOVE  = tgetstr(TTY_CURSOR_MOVE_NAME,  &capability_buf);
+
+    for (i = 0; i < TTY_USED_CAPABILITIES; i++)
+	if (tty_capability[i].capability == NULL)
+	    if (tty_capability[i].indispensable)
+	    {
+		term_errors++;
+		fprintf(stderr,
+			"%s: can't find the '%s' terminal capability.\n",
+			program, tty_capability[i].name);
+	    }
+
+    if (term_errors)
+    {
+	fprintf(stderr, "%s: %d errors. Your terminal is too dumb :-< .\n",
+		program, term_errors);
+	exit(1);
+    }
+}
