@@ -22,7 +22,12 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 #define _TTY_H
 
 
-#include "termcap.h"
+#define MAX_TTY_COLUMNS		512
+#define MAX_TTY_ROWS		512
+
+
+#define	MAX_CAPABILITY_LEN	 64	/* If you have greater terminal
+					   capabilities change this */
 
 
 #define OFF			0
@@ -38,15 +43,23 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 #define WHITE			7
 
 
-#define KEY_NOKEY		0000		/* No key (error)	*/
+#define key_NOKEY		0000		/* No key (error)	*/
 
-#define KEY_CTRL_H		0010		/* Ctrl-H		*/
-#define KEY_TAB 	       	0011		/* Tab			*/
-#define KEY_ENTER       	0012           	/* Enter		*/
-#define KEY_ESC			0033		/* Esc			*/
-#define KEY_SPACE		0040		/* Space		*/
+#define key_CTRL_H		0010		/* Ctrl-H		*/
+#define key_TAB 	       	0011		/* Tab			*/
+#define key_ENTER       	0012           	/* Enter		*/
+#define key_ESC			0033		/* Esc			*/
+#define key_SPACE		0040		/* Space		*/
 
-#define KEY_BACKSPACE   	0177    	/* Backspace		*/
+#define key_BACKSPACE   	0177    	/* Backspace		*/
+
+
+#define TTY_CANONIC	0
+#define TTY_NONCANONIC	1
+
+
+#define TTY_RESTRICTED_INPUT	0
+#define TTY_FULL_INPUT		1
 
 
 #define is_print(c)		((c) >= ' ' && (c) <= '~')
@@ -66,15 +79,18 @@ typedef struct
     int	bg_color;
     int	br_status;
     int	rv_status;
+    int cursor_status;
     unsigned char attribute;
 } tty_status;
 
 
 void tty_startup(void);
 void tty_exit(void);
-void tty_init(void);
-void tty_end(void);
-void tty_kbdinit(int mode);
+
+void tty_get_capabilities(void);
+void tty_kbdinit(int kbd_mode);
+void tty_set_mode(int mode);
+
 void tty_clrscr(void);
 void tty_touch(void);
 void tty_cursormove(int y, int x);
@@ -83,7 +99,7 @@ void tty_foreground(int color);
 void tty_background(int color);
 void tty_bright(int status);
 void tty_reverse(int status);
-void tty_blink(int status);
+void tty_cursor(int status);
 void tty_beep(void);
 void tty_defaults(void);
 void tty_save(tty_status *status);
@@ -98,7 +114,10 @@ void tty_putscreen(char *buf);
 int  tty_getcolorindex(char *colorname);
 void tty_key_list_insert(char *key_seq, void *aux_data);
 void tty_key_search_restart(void);
+/*
 void tty_key_list_delete(void);
+*/
+
 
 struct key_struct *tty_key_search(char *key_seq);
 struct key_struct *tty_getkey(int *repeat_count);
