@@ -20,16 +20,19 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <malloc.h>
 #include <limits.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <malloc.h>
 
 
 #define CMP_BUFFER_SIZE		64*1024
 #define min(a, b) ((a) <= (b) ? (a) : (b))
 
 
+char *program;
+
+
 int filelength(int handle)
 {
     int temp, length;
@@ -47,37 +50,35 @@ int main(int argc, char *argv[])
     char *buf1, *buf2;
     int handle1, handle2, i, j, len, len1, len2, bytes_to_compare;
 
+    program = argv[0];
+
     if (argc != 3)
     {
-        fprintf(stderr, "Invalid command line !\n");
+        fprintf(stderr, "%s: invalid command line.\n", program);
         return 1;
     }
 
     if ((handle1 = open(argv[1], O_RDONLY)) == -1)
     {
-        fprintf(stderr, "Can't open file %s !\n", argv[1]);
+        fprintf(stderr, "%s: can't open file '%s'.\n", program, argv[1]);
         return 1;
     }
 
     if ((handle2 = open(argv[2], O_RDONLY)) == -1)
     {
-        close(handle2);
-        fprintf(stderr, "Can't open file %s !\n", argv[2]);
+        fprintf(stderr, "%s: can't open file '%s'.\n", program, argv[2]);
         return 1;
     }
 
     if ((len1 = filelength(handle1)) != (len2 = filelength(handle2)))
-        fprintf(stderr, "Files have different sizes !\n");
+        fprintf(stderr, "%s: files have different sizes.\n", program);
 
     len = min(len1, len2);
 
     if ((buf1 = (char *)malloc(CMP_BUFFER_SIZE)) == NULL ||
         (buf2 = (char *)malloc(CMP_BUFFER_SIZE)) == NULL)
     {
-        close(handle1);
-        close(handle2);
-        if (buf1) free(buf1);
-        fprintf(stderr, "Not enough memory !\n");
+        fprintf(stderr, "%s: not enough memory.\n", program);
 	return 1;
     }
 
@@ -87,21 +88,15 @@ int main(int argc, char *argv[])
 
         if (read(handle1, buf1, bytes_to_compare) != bytes_to_compare)
         {
-            close(handle1);
-            close(handle2);
-            free(buf1);
-            free(buf2);
-            fprintf(stderr, "Can't read from file %s !\n", argv[1]);
+            fprintf(stderr, "%s: can't read from file %s.\n",
+            	    program, argv[1]);
 	    return 1;
         }
 
         if (read(handle2, buf2, bytes_to_compare) != bytes_to_compare)
         {
-            close(handle1);
-            close(handle2);
-            free(buf1);
-            free(buf2);
-            fprintf(stderr, "Can't read from file %s !\n", argv[2]);
+            fprintf(stderr, "%s: can't read from file %s.\n",
+            	    program, argv[2]);
             return 1;
         }
 
@@ -109,15 +104,12 @@ int main(int argc, char *argv[])
             for (j = 0; j < bytes_to_compare; j++)
                 if (buf1[j] != buf2[j])
                 {
-		    fprintf(stderr, "Filles differ at offset %d !\n", j);
+		    fprintf(stderr, "%s: filles differ at offset %d.\n",
+		    	    program, j);
 		    return 1;
 		}
     }
 
     fprintf(stderr, " Compare OK\n");
-    close(handle1);
-    close(handle2);
-    free(buf1);
-    free(buf2);
     return 1;
 }
