@@ -19,12 +19,16 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 
 
 #include <stdio.h>
+
 #include "fnmatch.h"
 
 
-int usage(char *prog)
+char *program;
+
+
+int usage(void)
 {
-    fprintf(stderr, "%s: UIT internal program\n", prog);
+    fprintf(stderr, "%s: UIT internal program.\n", program);
     exit(255);
 }
 
@@ -39,8 +43,10 @@ int main(int argc, char *argv[])
 {
     int i, flags = FNM_PERIOD | FNM_PATHNAME;
 
+    program = argv[0];
+
     if (argc < 3)
-	usage(argv[0]);
+	usage();
 
     for (i = 0; i < argc - 2; i++)
 	if (fnmatch(argv[i + 2], argv[1], flags) != FNM_NOMATCH)
