@@ -20,7 +20,6 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <malloc.h>
 #include <limits.h>
 #include <fcntl.h>
 #include <unistd.h>
@@ -29,24 +28,22 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 #include <errno.h>
 #include <termios.h>
 
+#include "xmalloc.h"
 #include "tty.h"
-#include "termcap.h"
 #include "window.h"
 #include "config.h"
-
-
-#define max(a, b) ((a) >= (b) ? (a) : (b))
-#define min(a, b) ((a) <= (b) ? (a) : (b))
+#include "tilde.h"
+#include "misc.h"
 
 
 int SCREEN_X, SCREEN_Y;
 
 #define MAX_KEYS        2048
 #define MAX_PROCESS	1024
-#define KILLER_FIELDS	  12
+#define PS_FIELDS	  12
 
 
-static char PSFields[KILLER_FIELDS][40] =
+static char *PSFields[PS_FIELDS] =
 {
     "TitleForeground",
     "TitleBackground",
@@ -63,16 +60,16 @@ static char PSFields[KILLER_FIELDS][40] =
 };
 
 #ifdef HAVE_LINUX
-static int PSColors[KILLER_FIELDS] = 
+static int PSColors[PS_FIELDS] = 
 {
     CYAN, BLUE, ON, CYAN, RED, ON, BLACK, CYAN, OFF, CYAN, BLUE, ON
 };
-#else
-static int PSColors[KILLER_FIELDS] = 
+#else	/* not HAVE_LINUX */
+static int PSColors[PS_FIELDS] = 
 {
     BLACK, WHITE, OFF, WHITE, BLACK, ON, WHITE, BLACK, OFF, BLACK, WHITE, OFF
 };
-#endif
+#endif	/* not HAVE_LINUX */
 
 #define TitleForeground			PSColors[0]
 #define TitleBackground			PSColors[1]
@@ -90,36 +87,36 @@ static int PSColors[KILLER_FIELDS] =
 
 
 #ifdef HAVE_LINUX
-int LinuxConsole = 1;
-int ColorMonitor = 1;
-#else
-int LinuxConsole = 0;
-int ColorMonitor = 0;
-#endif
-
-
-extern char tty_type[];
+int AnsiColorSequences = ON;
+#else	/* not HAVE_LINUX */
+int AnsiColorSequences = OFF;
+#endif	/* not HAVE_LINUX */
 
 
 char cSection[]  = "[UITPS-Color]";
 char bwSection[] = "[UITPS-Monochrome]";
-int processes;
-int PID_index;
-int signal_type = 11;	/* index of SIGTERM in signals table */
+int  processes;
+int  PID_index;
+int  signal_type = 11;	/* index of SIGTERM in signals table */
 
-char homedirectory[PATH_MAX];
-char configfile[PATH_MAX] = "";
-char TempDirectory[PATH_MAX];
+char *home;
+char *program;
+char *tty_name;
+int  tty_name_len;
 char *header_text;
-configuration *config;
+char *TempDirectory;
+char *stdout_log_name = NULL;
+char *stderr_log_name = NULL;
 char *ps_vect[MAX_PROCESS];
+char *global_buf = NULL;
 int first_on_screen, current_process;
 window *title_win, *header_win, *screen_win, *status_win;
-static char title_text[]  =
-    " UNIX Interactive Tools 4.3 - Process Viewer/Killer";
-static char UitPsModeHelp[256];
-static char no_perm[]     = "not owner !";
-static char no_proc[]     = "no such process ! (REFRESH recommended)";
+static char title_text[] =
+    " UNIX Interactive Tools 4.3.1 - Process Viewer/Killer";
+static char *UitPsModeHelp;
+static char no_perm[] = "not owner !";
+static char no_proc[] = "no such process ! (REFRESH recommended)";
+
 
 struct SIGNAL
 {
@@ -214,63 +211,54 @@ char built_in[BUILTIN_OPERATIONS][MAX_BUILTIN_NAME] =
 
 void removelog(void)
 {
-    char name[PATH_MAX];
-
-    sprintf(name, "%s/uitps.1.%d", TempDirectory, getpid());
-    remove(name);
-    sprintf(name, "%s/uitps.2.%d", TempDirectory, getpid());
-    remove(name);
+    if (stdout_log_name) remove(stdout_log_name);
+    if (stderr_log_name) remove(stderr_log_name);
 }
 
 
 void settitle(void)
 {
-    char temp[256];
-
-    memset(temp, ' ', SCREEN_X);
-    memcpy(temp, title_text, strlen(title_text));
+    memset(global_buf, ' ', SCREEN_X);
+    memcpy(global_buf, title_text, strlen(title_text));
 
     tty_bright(TitleBrightness);
     tty_foreground(TitleForeground);
     tty_background(TitleBackground);
 
     window_cursormove_notify(title_win, 0, 0);
-    window_write(temp, SCREEN_X);
+    window_write(global_buf, SCREEN_X);
 }
 
 
 void setheader(void)
 {
-    char temp[256];
-
-    memset(temp, ' ', SCREEN_X);
-    memcpy(temp, header_text, strlen(header_text));
+    memset(global_buf, ' ', SCREEN_X);
+    memcpy(global_buf, header_text, strlen(header_text));
 
     tty_bright(HeaderBrightness);
     tty_foreground(HeaderForeground);
     tty_background(HeaderBackground);
 
     window_cursormove_notify(header_win, 0, 0);
-    window_write(temp, SCREEN_X);
+    window_write(global_buf, SCREEN_X);
 }
 
 
 void setstatus(char *what)
 {
-    char temp[256];
+    memset(global_buf, ' ', SCREEN_X);
 
-    memset(temp, ' ', SCREEN_X);
     if (what)
-	memcpy(temp, what, strlen(what));
+	memcpy(global_buf, what, strlen(what));
     else
-	memcpy(temp, UitPsModeHelp, strlen(UitPsModeHelp));
+	memcpy(global_buf, UitPsModeHelp, strlen(UitPsModeHelp));
 
     tty_bright(StatusBrightness);
     tty_foreground(StatusForeground);
     tty_background(StatusBackground);
 
     window_cursormove_notify(status_win, 0, 0);
-    window_write(temp, SCREEN_X - (sizeof(signals[0].signame) - 1) - 1);
+    window_write(global_buf, SCREEN_X - (sizeof(signals[0].signame) - 1) - 1);
 }
 
 
@@ -360,7 +348,7 @@ void build_ps_list(FILE *ps_output)
     int i = 0;
 
     do
-	ps_vect[i] = (char *)malloc(SCREEN_X + 1);
+	ps_vect[i] = xmalloc(SCREEN_X + 1);
     while (read_ps_line(ps_output, ps_vect[i++]));
 
     free(ps_vect[--i]);
@@ -371,13 +359,13 @@ void build_ps_list(FILE *ps_output)
 
 void update_process(int process, int update_color)
 {
-    char temp[256];
-
-    memset(temp, ' ', SCREEN_X);
-    memcpy(temp, ps_vect[process], strlen(ps_vect[process]));
+    memset(global_buf, ' ', SCREEN_X);
+    memcpy(global_buf, ps_vect[process], strlen(ps_vect[process]));
 
     if (update_color)
     {
+	tty_bright(ScreenBrightness);
+
 	if (process == current_process)
 	{
 	    tty_foreground(ScreenBackground);
@@ -388,25 +376,24 @@ void update_process(int process, int update_color)
 	    tty_foreground(ScreenForeground);
 	    tty_background(ScreenBackground);
 	}
-	tty_bright(ScreenBrightness);
     }
 
     window_cursormove_notify(screen_win, process - first_on_screen, 0);
-    window_write(temp, SCREEN_X);
+    window_write(global_buf, SCREEN_X);
 }
 
 
 void update_all(void)
 {
     int i;
-    char fill[256];
 
+    tty_bright(ScreenBrightness);
     tty_foreground(ScreenForeground);
     tty_background(ScreenBackground);
-    tty_bright(ScreenBrightness);
-    window_cursormove(screen_win, 0, 0);
-    memset(fill, ' ', SCREEN_X);
 
+    tty_cursor(OFF);
+
+    window_cursormove(screen_win, 0, 0);
     for (i = first_on_screen;
          i < processes && (i - first_on_screen < SCREEN_Y - 3); i++)
 	    if (i != current_process)
@@ -416,31 +403,25 @@ void update_all(void)
 
     update_process(current_process, ON);
 
-    tty_background(ScreenBackground);
     tty_bright(ScreenBrightness);
+    tty_foreground(ScreenForeground);
+    tty_background(ScreenBackground);
+
+    memset(global_buf, ' ', SCREEN_X);
 
     for (; i - first_on_screen < SCREEN_Y - 3; i++)
     {
 	window_cursormove_notify(screen_win, i - first_on_screen, 0);
-	window_write(fill, SCREEN_X);
+	window_write(global_buf, SCREEN_X);
     }
 
     window_cursormove(screen_win, current_process-first_on_screen, SCREEN_X-1);
+    tty_cursor(ON);
 }
 
 
 void clean_up(void)
 {
-    free_ps_list();
-
-    if (header_text)
-    {
-	free(header_text);
-	header_text = NULL;
-    }
-
-    tty_end();
-    tty_clrscr();
     tty_exit();
     removelog();
 }
@@ -449,22 +430,22 @@ void clean_up(void)
 void fatal(char *postmsg)
 {
     clean_up();
-    fprintf(stderr, "Fatal error : %s !\n", postmsg);
+    fprintf(stderr, "%s: fatal error: %s.\n", program, postmsg);
     exit(1);    
 }
 
 
 int ps(char *args)
 {
+    char *ps_cmd;
     FILE *stdout_log, *stderr_log;
-    char ps_cmd[PATH_MAX], fname[PATH_MAX], *tty_name = ttyname(1);
 
     close(1);
     close(2);
-    sprintf(fname, "%s/uitps.1.%d", TempDirectory, getpid());
-    stdout_log = fopen(fname, "w");
-    sprintf(fname, "%s/uitps.2.%d", TempDirectory, getpid());
-    stderr_log = fopen(fname, "w");
+    stdout_log = fopen(stdout_log_name, "w");
+    stderr_log = fopen(stderr_log_name, "w");
+
+    ps_cmd = xmalloc(16 + strlen(args) + 1);
 
     if (args)
 	sprintf(ps_cmd, "exec ps %s", args);
@@ -477,10 +458,11 @@ int ps(char *args)
 	fclose(stdout_log);
 	open(tty_name, O_RDWR);
 	open(tty_name, O_RDWR);
-	fprintf(stderr, "Invalid command line !\n");
+	fprintf(stderr, "%s: invalid command line.\n", program);
 	return 0;
     }
 
+    free(ps_cmd);
     fclose(stderr_log);
     fclose(stdout_log);
     open(tty_name, O_RDWR);
@@ -492,37 +474,32 @@ int ps(char *args)
 void panic(int signum)
 {
     tcflush(0, TCIOFLUSH);
+    clean_up();
 
     switch (signum)
     {
 	case SIGHUP:
 
-	    clean_up();
 	    /* :-) */
-	    fprintf(stderr, "Got SIGHUP !\n");
+	    fprintf(stderr, "%s: got SIGHUP.\n", program);
 	    break;
 
 	case SIGINT:
 
-	    clean_up();
 	    break;
 
 	case SIGTERM:
 	case SIGQUIT:
 
-	    clean_up();
-	    fprintf(stderr, "Got %s !\nbye\n",
-	    	   (signum == SIGTERM) ? "SIGTERM" : "SIGQUIT");
+	    fprintf(stderr, "%s: got %s.\n",
+	    	   program, ((signum == SIGTERM) ? "SIGTERM" : "SIGQUIT"));
 	    break;
 
 	case SIGSEGV:
 
-	    tty_end();
-	    tty_clrscr();
-	    tty_exit();
-	    removelog();
-	    fprintf(stderr, "Got SIGSEGV !\n");
-	    fprintf(stderr, "Please report to tudor@ulise.cs.pub.ro\n");
+	    fprintf(stderr, "%s: got SIGSEGV.\n", program);
+	    fprintf(stderr, "%s: please report to tudor@ulise.cs.pub.ro.\n",
+	    	    program);
 	    break;
     }
 
@@ -532,17 +509,27 @@ void panic(int signum)
 
 int main(int argc, char *argv[])
 {
+    char *contents;
+    char key_seq[16];
     FILE *stdout_log;
+    char *tmp, *buf;
     struct key_struct *ks;
     int key, repeat_count;
-    char tmp[256], buf[256];
-    int sectionptr = 0, i, j, index, first_time = 1;
-    char *home, *data = NULL, fname[PATH_MAX], *section;
+    int i, j, index, first_time = 1;
+    char *data = NULL;
     int need_update, need_update_all, old_current_process;
 
 
-    termcap_init();
-    tty_kbdinit(1);
+    program = argv[0];
+
+    home = getenv("HOME");
+    if (!home) home = ".";
+
+    get_tty_name();
+    get_login_name();
+
+    tty_get_capabilities();
+    tty_kbdinit(TTY_FULL_INPUT);
 
     signal(SIGTERM, panic);
     signal(SIGINT , panic);
@@ -561,123 +548,75 @@ int main(int argc, char *argv[])
     signal(SIGPIPE, SIG_IGN);
     signal(SIGFPE,  SIG_IGN);
 
-    if (home = getenv("HOME"))
+    configuration_check();
+
+
+    use_section("[Setup]");
+
+    configuration_getvarinfo("TempDirectory", &data, 1, DO_SEEK);
+    TempDirectory = data ? tilde_expand(data) : "/tmp";
+
+    AnsiColorSequences = get_flag_var("AnsiColorSequences", OFF);
+
+
+    use_section("[UITPS-Setup]");
+
+    UitPsModeHelp = get_string_var("UitPsModeHelp", "");
+
+
+    use_section(AnsiColorSequences ? cSection : bwSection);
+
+    get_colorset_var(PSColors, PSFields, PS_FIELDS);
+
+
+    use_section("[UITPS-Keys]");
+
+    for (i = 0; i < MAX_KEYS; i++)
     {
-        strcpy(homedirectory, home);
-        strcpy(configfile, home);
-        if (configfile[strlen(configfile) - 1] != '/') strcat(configfile, "/");
-    }
-    strcat(configfile, ".uitrc.");
-    strcat(configfile, tty_type);
+        configuration_getvarinfo(key_seq, &contents, 1, NO_SEEK);
 
-    config = configuration_init(configfile);
+	if (*key_seq == 0) break;
 
-    if (configuration_getstatus(config) == STATUS_OK)
-    {
-	sectionptr = configuration_getsectionptr(config,"[Setup]");
-        if (sectionptr == -1)
-	    puts("[Setup] section missing.");
-        else
-        {
-	    configuration_getfielddata(config, sectionptr, "TempDirectory",
-				       &data, 1, DO_SEEK);
-	    strncpy(TempDirectory, data ? data : homedirectory, PATH_MAX);
-#ifdef HAVE_LINUX
-	    configuration_getfielddata(config, sectionptr, "LinuxConsole",
-				       &data, 1, DO_SEEK);
-	    LinuxConsole = data && strcmp(data, "ON") == 0;
-	    configuration_getfielddata(config, sectionptr, "ColorMonitor",
-				       &data, 1, DO_SEEK);
-	    ColorMonitor = data && strcmp(data, "ON") == 0;
-#endif
-	}
+	if (contents == NULL) continue;
 
-	sectionptr = configuration_getsectionptr(config, "[UITPS-Setup]");
-        if (sectionptr == -1)
-	    fprintf(stderr, "[UITPS-Setup] section missing.");
-	else
+        for (j = 0; j < BUILTIN_OPERATIONS; j++)
+            if (strcmp(contents, built_in[j]) == 0)
+                break;
+
+	if (j < BUILTIN_OPERATIONS)
 	{
-	    configuration_getfielddata(config, sectionptr,
-				       "UitPsModeHelp",
-				       &data, 1, DO_SEEK);
-	    if (data) strncpy(UitPsModeHelp, data, 256);
+	    if (tty_key_convert(key_seq))
+	        tty_key_list_insert(key_seq, (void *)j);
 	}
-
-	section = (LinuxConsole && ColorMonitor) ? cSection : bwSection;
-
-	sectionptr = configuration_getsectionptr(config, section);
-	if (sectionptr == -1)
-	    fprintf(stderr, "%s section missing.", section);
 	else
-	    for (i = 0; i < KILLER_FIELDS; i++)
-	    {
-		configuration_getfielddata(config, sectionptr, PSFields[i],
-					   &data, 1, DO_SEEK);
-        	if (!data || (index  = tty_getcolorindex(data))== -1)
-                    fprintf(stderr, "Invalid %s (%s).\n", PSFields[i], data);
-		else
-		    PSColors[i] = index;
-	    }
-
-        sectionptr = configuration_getsectionptr(config, "[UITPS-Keys]");
-        if (sectionptr == -1)
-	    fprintf(stderr, "[UITPS-Keys] section missing.");
-	else
-	{
-	    char key_seq[16];
-	    char *contents;
-
-	    for (i = 0; i < MAX_KEYS; i++)
-	    {
-	        configuration_getfielddata(config, sectionptr, key_seq,
-					   &contents, 1, NO_SEEK);
-
-		if (*key_seq == 0) break;
-
-		if (contents == NULL) continue;
-
-	        for (j = 0; j < BUILTIN_OPERATIONS; j++)
-	            if (strcmp(contents, built_in[j]) == 0)
-	                break;
-
-		if (j < BUILTIN_OPERATIONS)
-		{
-		    if (tty_key_convert(key_seq))
-		        tty_key_list_insert(key_seq, (void *)j);
-		}
-		else
-		    fprintf(stderr, "Invalid built-in operation: %s\n",
-			    contents);
-	    }
-
-	    if (i == MAX_KEYS)
-	        fprintf(stderr,
-			"Too many key sequences. Only %d are allowed.\n",
-			MAX_KEYS);
-	}
-	configuration_end(config);
-    }
-    else
-    {
-        fprintf(stderr, "\nCannot open configuration file %s\n", configfile);
-	fprintf(stderr, "See the manual page for details.\n");
-	fprintf(stderr, "If your TERM environment variable is, let's say,\n");
-	fprintf(stderr, "vt102, your configuration file name should be\n");
-	fprintf(stderr, ".uitrc.vt102 . Try renaming .uitrc.console ...\n");
-	exit(1);
+	    fprintf(stderr, "%s: invalid built-in operation: %s.\n",
+		    program, contents);
     }
 
+    if (i == MAX_KEYS)
+        fprintf(stderr,	"%s: too many key sequences; only %d are allowed.\n",
+		program, MAX_KEYS);
+
+    configuration_end();
+
     tty_getsize(&SCREEN_X, &SCREEN_Y);
     tty_startup();
 
-    header_text = (char *)malloc(SCREEN_X + 1);
+    stdout_log_name = xmalloc(32 + strlen(TempDirectory) + 1);
+    stderr_log_name = xmalloc(32 + strlen(TempDirectory) + 1);
+    sprintf(stdout_log_name, "%s/uitps.1.%d", TempDirectory, getpid());
+    sprintf(stderr_log_name, "%s/uitps.2.%d", TempDirectory, getpid());
+
+    global_buf  = xmalloc(SCREEN_X + 1);
+
+    header_text = xmalloc(SCREEN_X + 1);
 
     title_win  = window_init(0, 0, 	      1,	    SCREEN_X);
     header_win = window_init(0, 1, 	      1,	    SCREEN_X);
     screen_win = window_init(0, 2,            SCREEN_Y - 3, SCREEN_X);
     status_win = window_init(0, SCREEN_Y - 1, 1,	    SCREEN_X);
 
-    tty_init();
+    tty_set_mode(TTY_NONCANONIC);
 
     first_on_screen = current_process = 0;
 
@@ -689,15 +628,11 @@ restart:
 	goto end;
     }
 
-    sprintf(fname, "%s/uitps.1.%d", TempDirectory, getpid());
-    stdout_log = fopen(fname, "r");
+    stdout_log = fopen(stdout_log_name, "r");
     removelog();
 
     if ((PID_index = get_PID_index(stdout_log)) == -1)
-    {
-	fclose(stdout_log);
 	goto end;
-    }
 
     free_ps_list();
     build_ps_list(stdout_log);
@@ -705,9 +640,9 @@ restart:
 
     if (first_time)
     {
+	tty_bright(ScreenBrightness);
 	tty_foreground(ScreenForeground);
 	tty_background(ScreenBackground);
-	tty_bright(ScreenBrightness);
 	tty_clrscr();
 	first_time = 0;
     }
@@ -888,9 +823,11 @@ restart:
 	        if (!kill_process(current_process))
 	        {
 	            tty_beep();
-	            memset(buf, ' ', SCREEN_X);
+	            memset(global_buf, ' ', SCREEN_X);
+	            tmp = xmalloc(16 + strlen((errno==EPERM)?no_perm:no_proc));
 		    sprintf(tmp, "Error: %s", (errno==EPERM)?no_perm:no_proc);
-		    memcpy(buf, tmp, strlen(tmp));
+		    memcpy(global_buf, tmp, strlen(tmp));
+		    free(tmp);
 		    setstatus(buf);
 		    errno = 0;
 		    tty_getkey(NULL);
