@@ -20,7 +20,6 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <malloc.h>
 #include <limits.h>
 #include <fcntl.h>
 #include <unistd.h>
@@ -31,14 +30,12 @@ UIT; see the file COPYING.  If not, write to the Free Software Foundation,
 #include <termios.h>
 #include <sys/stat.h>
 
+#include "xmalloc.h"
 #include "tty.h"
-#include "termcap.h"
 #include "window.h"
 #include "config.h"
-
-
-#define max(a, b) ((a) >= (b) ? (a) : (b))
-#define min(a, b) ((a) <= (b) ? (a) : (b))
+#include "tilde.h"
+#include "misc.h"
 
 
 #define MAX_KEYS		2048
@@ -77,7 +74,7 @@ int SCREEN_X, SCREEN_Y;
 
 #define VIEWER_FIELDS 12
 
-static char VIEWERFields[VIEWER_FIELDS][40] = 
+static char *ViewerFields[VIEWER_FIELDS] =
 {
     "TitleForeground",
     "TitleBackground",
@@ -94,56 +91,53 @@ static char VIEWERFields[VIEWER_FIELDS][40] =
 };
 
 #ifdef HAVE_LINUX
-static int VIEWERColors[VIEWER_FIELDS] = 
+static int ViewerColors[VIEWER_FIELDS] = 
 {
     CYAN, BLUE, ON, CYAN, RED, ON, BLACK, CYAN, OFF, CYAN, BLUE, ON
 };
-#else
-static int VIEWERColors[VIEWER_FIELDS] = 
+#else	/* not HAVE_LINUX */
+static int ViewerColors[VIEWER_FIELDS] = 
 {
     BLACK, WHITE, OFF, WHITE, BLACK, ON, WHITE, BLACK, OFF, BLACK, WHITE, OFF
 };
-#endif
+#endif	/* not HAVE_LINUX */
 
-#define TitleForeground			VIEWERColors[0]
-#define TitleBackground			VIEWERColors[1]
-#define TitleBrightness			VIEWERColors[2]
-#define HeaderForeground		VIEWERColors[3]
-#define HeaderBackground		VIEWERColors[4]
-#define HeaderBrightness		VIEWERColors[5]
-#define ScreenForeground		VIEWERColors[6]
-#define ScreenBackground		VIEWERColors[7]
-#define ScreenBrightness		VIEWERColors[8]
-#define StatusForeground		VIEWERColors[9]
-#define StatusBackground		VIEWERColors[10]
-#define StatusBrightness		VIEWERColors[11]
+#define TitleForeground			ViewerColors[0]
+#define TitleBackground			ViewerColors[1]
+#define TitleBrightness			ViewerColors[2]
+#define HeaderForeground		ViewerColors[3]
+#define HeaderBackground		ViewerColors[4]
+#define HeaderBrightness		ViewerColors[5]
+#define ScreenForeground		ViewerColors[6]
+#define ScreenBackground		ViewerColors[7]
+#define ScreenBrightness		ViewerColors[8]
+#define StatusForeground		ViewerColors[9]
+#define StatusBackground		ViewerColors[10]
+#define StatusBrightness		ViewerColors[11]
 
 
 #ifdef HAVE_LINUX
-int LinuxConsole = 1;
-int ColorMonitor = 1;
-#else
-int LinuxConsole = 0;
-int ColorMonitor = 0;
-#endif
+int AnsiColorSequences = ON;
+#else	/* not HAVE_LINUX */
+int AnsiColorSequences = OFF;
+#endif	/* not HAVE_LINUX */
 
 
-extern char tty_type[];
 
-
-char homedirectory[PATH_MAX];
-char configfile[PATH_MAX] = "";
+char *home;
+char *program;
+char *tty_name;
+int  tty_name_len;
+char *header_text;
+char *global_buf = NULL;
 char cSection[]  = "[UITVIEW-Color]";
 char bwSection[] = "[UITVIEW-Monochrome]";
-
-char *header_text;
-configuration *config;
 int handle, regular_file;
 int current_line, lines;
 window *title_win, *header_win, *screen_win, *status_win;
 static char title_text[]=
-    " UNIX Interactive Tools 4.3 - Hex/Ascii File Viewer";
-static char UitViewModeHelp[256];
+    " UNIX Interactive Tools 4.3.1 - Hex/Ascii File Viewer";
+static char *UitViewModeHelp;
 static char info_txt[] =
     "   Offset    00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F        \
 Ascii       ";
@@ -168,49 +162,43 @@ int filelength(void)
 
 void settitle(void)
 {
-    char temp[256];
-
-    memset(temp, ' ', SCREEN_X);
-    memcpy(temp, title_text, strlen(title_text));
+    memset(global_buf, ' ', SCREEN_X);
+    memcpy(global_buf, title_text, strlen(title_text));
 
     tty_bright(TitleBrightness);
     tty_foreground(TitleForeground);
     tty_background(TitleBackground);
 
     window_cursormove_notify(title_win, 0, 0);
-    window_write(temp, SCREEN_X);
+    window_write(global_buf, SCREEN_X);
 }
 
 
 void setheader(void)
 {
-    char temp[256];
-
-    memset(temp, ' ', SCREEN_X);
-    memcpy(temp, header_text, min(strlen(header_text), SCREEN_X));
+    memset(global_buf, ' ', SCREEN_X);
+    memcpy(global_buf, header_text, min(strlen(header_text), SCREEN_X));
 
     tty_bright(HeaderBrightness);
     tty_foreground(HeaderForeground);
     tty_background(HeaderBackground);
 
     window_cursormove_notify(header_win, 0, 0);
-    window_write(temp, SCREEN_X);
+    window_write(global_buf, SCREEN_X);
 }
 
 
 void setstatus(void)
 {
-    char temp[256];
-
-    memset(temp, ' ', SCREEN_X);
-    memcpy(temp, UitViewModeHelp, strlen(UitViewModeHelp));
+    memset(global_buf, ' ', SCREEN_X);
+    memcpy(global_buf, UitViewModeHelp, strlen(UitViewModeHelp));
 
     tty_bright(StatusBrightness);
     tty_foreground(StatusForeground);
     tty_background(StatusBackground);
 
     window_cursormove_notify(status_win, 0, 0);
-    window_write(temp, SCREEN_X);	
+    window_write(global_buf, SCREEN_X);	
 }
 
 
@@ -223,15 +211,15 @@ char char_to_print(char c)
 void update_line(int line)
 {
     int r;
-    unsigned char temp[256], ln[16];
+    unsigned char ln[16];
 
     memset(ln, 0, 16);
-    memset(temp, ' ', SCREEN_X);
+    memset(global_buf, ' ', SCREEN_X);
     lseek(handle, line * 16, SEEK_SET);
 
     if ((r = read(handle, ln, 16)))
-	sprintf(temp, "  %07X0   %02X %02X %02X %02X %02X %02X %02X %02X %02X\
- %02X %02X %02X %02X %02X %02X %02X  %c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c  ",
+	sprintf(global_buf, "  %07X0   %02X %02X %02X %02X %02X %02X %02X\
+ %02X %02X %02X %02X %02X %02X %02X %02X %02X  %c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c  ",
 		line,
 		ln[0], ln[1], ln[2],  ln[3],  ln[4],  ln[5],  ln[6],  ln[7],
 		ln[8], ln[9], ln[10], ln[11], ln[12], ln[13], ln[14], ln[15],
@@ -246,11 +234,11 @@ void update_line(int line)
 
     if (r != 16)
     {
-	memset(temp + 13 + r * 3, ' ', (16 - r) * 3 - 1);
-	memset(temp + 13 + 16 * 3 - 1 + 2 + r, ' ', 16 - r + 2);
+	memset(global_buf + 13 + r * 3, ' ', (16 - r) * 3 - 1);
+	memset(global_buf + 13 + 16 * 3 - 1 + 2 + r, ' ', 16 - r + 2);
     }
 
-    window_write((char *)temp, SCREEN_X);
+    window_write((char *)global_buf, SCREEN_X);
 }
 
 
@@ -258,28 +246,24 @@ void update_all(void)
 {
     int i;
     
+    tty_cursor(OFF);
+
+    tty_bright(ScreenBrightness);
     tty_foreground(ScreenForeground);
     tty_background(ScreenBackground);
-    tty_bright(ScreenBrightness);
 
     for (i = current_line; i < current_line + 16; i++)
     {
         window_cursormove_notify(screen_win, 3 + i - current_line, 0);
 	update_line(i);
     }
+
+    tty_cursor(ON);
 }
 
 
 void clean_up(void)
 {
-    if (header_text)
-    {
-	free(header_text);
-	header_text = NULL;
-    }
-
-    tty_end();
-    tty_clrscr();
     tty_exit();
 }
 
@@ -287,7 +271,7 @@ void clean_up(void)
 void fatal(char *postmsg)
 {
     clean_up();
-    fprintf(stderr, "Fatal error : %s !\n", postmsg);
+    fprintf(stderr, "%s: fatal error: %s.\n", program, postmsg);
     exit(1);
 }
 
@@ -295,36 +279,32 @@ void fatal(char *postmsg)
 void panic(int signum)
 {
     tcflush(0, TCIOFLUSH);
+    clean_up();
 
     switch (signum)
     {
 	case SIGHUP:
 
-	    clean_up();
 	    /* :-) */
-	    fprintf(stderr, "Got SIGHUP !\n");
+	    fprintf(stderr, "%s: got SIGHUP.\n", program);
 	    break;
 
 	case SIGINT:
 
-	    clean_up();
 	    break;
 
 	case SIGTERM:
 	case SIGQUIT:
 
-	    clean_up();
-	    fprintf(stderr, "Got %s !\nbye\n",
-	    	   (signum == SIGTERM) ? "SIGTERM" : "SIGQUIT");
+	    fprintf(stderr, "%s: got %s.\n",
+		    program, ((signum == SIGTERM) ? "SIGTERM" : "SIGQUIT"));
 	    break;
 
 	case SIGSEGV:
 
-	    tty_end();
-	    tty_clrscr();
-	    tty_exit();
-	    fprintf(stderr, "Got SIGSEGV !\n");
-	    fprintf(stderr, "Please report to tudor@ulise.cs.pub.ro\n");
+	    fprintf(stderr, "%s: got SIGSEGV.\n", program);
+	    fprintf(stderr, "%s: please report to tudor@ulise.cs.pub.ro.\n",
+	    	    program);
 	    break;
     }
 
@@ -335,16 +315,28 @@ void panic(int signum)
 int main(int argc, char *argv[])
 {
     struct stat s;
+    char *contents;
     int need_update;
+    char key_seq[16];
+    char offset[16];
+    char *data = NULL;
     struct key_struct *ks;
-    char *home, *data = NULL, *section;
-    char tmp[256], buf[256], offset[16];
-    int sectionptr = 0, i, j, index, cnt = 0;
+    int i, j, index, cnt = 0;
     int key, repeat_count, size, first_time = 1;
 
 
-    termcap_init();
-    tty_kbdinit(0);
+    program = argv[0];
+
+    home = getenv("HOME");
+    if (!home) home = ".";
+
+    get_tty_name();
+    get_login_name();
+
+    tty_name_len = strlen(tty_name);
+
+    tty_get_capabilities();
+    tty_kbdinit(TTY_RESTRICTED_INPUT);
 
     signal(SIGTERM, panic);
     signal(SIGINT , panic);
@@ -365,131 +357,75 @@ int main(int argc, char *argv[])
 
     if (argc != 2)
     {
-	fprintf(stderr, "Usage: uitview filename\n");
+	fprintf(stderr, "usage: uitview filename\n");
 	exit(1);
     }
 
     stat(argv[1], &s);
+
     if (!(S_ISREG(s.st_mode) || S_ISBLK(s.st_mode)))
     {
-	fprintf(stderr, "%s is neither regular file nor block device.\n",
-		argv[1]);
+	fprintf(stderr, "%s: %s is neither regular file nor block device.\n",
+		program, argv[1]);
 	exit(1);
     }
 
     handle = open(argv[1], O_RDONLY);
+
     if (handle == -1)
     {
-	fprintf(stderr, "Cannot open file %s\n", argv[1]);
+	fprintf(stderr, "%s: cannot open file %s.\n", program, argv[1]);
 	exit(1);
     }
 
     regular_file = S_ISREG(s.st_mode);
 
-    if (home = getenv("HOME"))
+    configuration_check();
+
+
+    use_section("[Setup]");
+
+    AnsiColorSequences = get_flag_var("AnsiColorSequences", OFF);
+
+
+    use_section("[UITVIEW-Setup]");
+
+    UitViewModeHelp = get_string_var("UitViewModeHelp", "");
+
+    use_section(AnsiColorSequences ? cSection : bwSection);
+
+    get_colorset_var(ViewerColors, ViewerFields, VIEWER_FIELDS);
+
+
+    use_section("[UITVIEW-Keys]");
+
+    for (i = 0; i < MAX_KEYS; i++)
     {
-        strcpy(homedirectory, home);
-        strcpy(configfile, home);
-        if (configfile[strlen(configfile) - 1] != '/') strcat(configfile, "/");
+        configuration_getvarinfo(key_seq, &contents, 1, NO_SEEK);
+
+	if (*key_seq == 0) break;
+
+	if (contents == NULL) continue;
+
+        for (j = 0; j < BUILTIN_OPERATIONS; j++)
+            if (strcmp(contents, built_in[j]) == 0)
+                break;
+
+	if (j < BUILTIN_OPERATIONS)
+	{
+	    if (tty_key_convert(key_seq))
+	        tty_key_list_insert(key_seq, (void *) -(j + 1));
+	}
+	else
+	    fprintf(stderr, "%s: invalid built-in operation: %s.\n",
+		    program, contents);
     }
 
-    strcat(configfile, ".uitrc.");
-    strcat(configfile, tty_type);
+    if (i == MAX_KEYS)
+        fprintf(stderr,	"%s: too many key sequences; only %d are allowed.\n",
+		program, MAX_KEYS);
 
-    config = configuration_init(configfile);
-
-    if (configuration_getstatus(config) == STATUS_OK)
-    {
-	sectionptr = configuration_getsectionptr(config,"[Setup]");
-	if (sectionptr == -1)
-	    puts("[Setup] section missing.");
-	else
-	{
-#ifdef HAVE_LINUX
-	    configuration_getfielddata(config, sectionptr, "LinuxConsole",
-				       &data, 1, DO_SEEK);
-	    LinuxConsole = data && strcmp(data, "ON") == 0;
-	    configuration_getfielddata(config, sectionptr, "ColorMonitor",
-				       &data, 1, DO_SEEK);
-	    ColorMonitor = data && strcmp(data, "ON") == 0;
-#endif
-	}
-
-	sectionptr = configuration_getsectionptr(config, "[UITVIEW-Setup]");
-        if (sectionptr == -1)
-	    fprintf(stderr, "[UITVIEW-Setup] section missing.");
-	else
-	{
-	    configuration_getfielddata(config, sectionptr,
-				       "UitViewModeHelp",
-				       &data, 1, DO_SEEK);
-	    if (data) strncpy(UitViewModeHelp, data, 256);
-	}
-
-	section = (LinuxConsole && ColorMonitor) ? cSection : bwSection;
-
-	sectionptr = configuration_getsectionptr(config, section);
-	if (sectionptr == -1)
-	    fprintf(stderr, "%s section missing.", section);
-	else
-	    for (i = 0; i < VIEWER_FIELDS; i++)
-	    {
-		configuration_getfielddata(config, sectionptr, VIEWERFields[i],
-					   &data, 1, DO_SEEK);
-		if (!data || (index  = tty_getcolorindex(data))== -1)
-		    fprintf(stderr, "Invalid %s (%s).\n", VIEWERFields[i],
-			    data);
-		else
-		    VIEWERColors[i] = index;
-	    }
-
-        sectionptr = configuration_getsectionptr(config, "[UITVIEW-Keys]");
-        if (sectionptr == -1)
-	    fprintf(stderr, "[UITVIEW-Keys] section missing.");
-	else
-	{
-	    char key_seq[16];
-	    char *contents;
-
-	    for (i = 0; i < MAX_KEYS; i++)
-	    {
-	        configuration_getfielddata(config, sectionptr, key_seq,
-					   &contents, 1, NO_SEEK);
-
-		if (*key_seq == 0) break;
-
-		if (contents == NULL) continue;
-
-	        for (j = 0; j < BUILTIN_OPERATIONS; j++)
-	            if (strcmp(contents, built_in[j]) == 0)
-	                break;
-
-		if (j < BUILTIN_OPERATIONS)
-		{
-		    if (tty_key_convert(key_seq))
-		        tty_key_list_insert(key_seq, (void *) -(j + 1));
-		}
-		else
-		    fprintf(stderr, "Invalid built-in operation: %s\n",
-			    contents);
-	    }
-
-	    if (i == MAX_KEYS)
-	        fprintf(stderr,
-			"Too many key sequences. Only %d are allowed.\n",
-			MAX_KEYS);
-	}
-	configuration_end(config);
-    }
-    else
-    {
-        fprintf(stderr, "\nCannot open configuration file %s\n", configfile);
-	fprintf(stderr, "See the manual page for details.\n");
-	fprintf(stderr, "If your TERM environment variable is, let's say,\n");
-	fprintf(stderr, "vt102, your configuration file name should be\n");
-	fprintf(stderr, ".uitrc.vt102 . Try renaming .uitrc.console ...\n");
-	exit(1);
-    }
+    configuration_end();
 
     tty_getsize(&SCREEN_X, &SCREEN_Y);
     tty_startup();
@@ -497,10 +433,13 @@ int main(int argc, char *argv[])
     if (SCREEN_Y < 24)
     {
 	SCREEN_Y = 24;
-	fprintf(stderr, "WARNING: can't use less than 24 columns.\n");
+	fprintf(stderr, "%s: WARNING: can't use less than 24 columns.\n",
+		program);
     }
 
-    header_text = (char *)malloc(strlen(argv[1]) + 10);
+    global_buf  = xmalloc(SCREEN_X + 1);
+
+    header_text = xmalloc(strlen(argv[1]) + 10);
     sprintf(header_text, " File: %s", argv[1]);
 
     title_win  = window_init(0, 0, 	      1,	    SCREEN_X);
@@ -508,7 +447,7 @@ int main(int argc, char *argv[])
     screen_win = window_init(0, 2,            SCREEN_Y - 2, SCREEN_X);
     status_win = window_init(0, SCREEN_Y - 1, 1,	    SCREEN_X);
 
-    tty_init();
+    tty_set_mode(TTY_NONCANONIC);
 
     offset[cnt]  = 0;
     current_line = 0;
@@ -518,9 +457,9 @@ restart:
 
     if (first_time)
     {
+	tty_bright(ScreenBrightness);
 	tty_foreground(ScreenForeground);
 	tty_background(ScreenBackground);
-	tty_bright(ScreenBrightness);
 	tty_clrscr();
 	first_time = 0;
     }
@@ -529,9 +468,9 @@ restart:
     setstatus();
     setheader();
 
+    tty_bright(ScreenBrightness);
     tty_foreground(ScreenForeground);
     tty_background(ScreenBackground);
-    tty_bright(ScreenBrightness);
 
     window_cursormove_notify(screen_win, 1, 0);
     window_write(info_txt, sizeof(info_txt) - 1);
@@ -644,7 +583,7 @@ restart:
 	            tty_beep();
 		break;
 
-	    case KEY_BACKSPACE:
+	    case key_BACKSPACE:
 
 	        if (cnt) cnt --;
 		window_cursormove_notify(screen_win, 20,
@@ -652,7 +591,7 @@ restart:
 		window_putch(' ');
 	        break;
 
-	    case KEY_ENTER:
+	    case key_ENTER:
 
 		if (cnt == 0)
 		    tty_beep();
